/**
    Copyright (C) 2001, 2003  Michael Ahlberg, Måns Rullgård

    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without
    restriction, including without limitation the rights to use, copy,
    modify, merge, publish, distribute, sublicense, and/or sell copies
    of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
**/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <tcconf.h>
#include <tcalloc.h>
#include <tcdirent.h>
#include <kernel.h>

#define DEFAULTCONFIG (SYSCONFDIR "/tc2.conf")

static tcconf_section_t *top_conf;
static int conf_init;
static tchash_table_t *files;
static char *conf_path;

static int
init_config(void)
{
    char cfg_file[1024];
    char *conf = getenv("TC2_CONFIG"), *cp;

    if(!conf)
	conf = DEFAULTCONFIG;
    top_conf = tcconf_load_file(NULL, conf);

    if(!top_conf){
	fprintf(stderr, "TC2: warning: error loading config file %s.\n"
		"TC2: Install a proper config file, or set the TC2_CONFIG\n"
		"TC2: environment variable to the full path of a proper file.\n"
		"TC2: Prepare for the worst.\n", conf);
    }

    
    if((cp = getenv("TC2_CONFIG_PATH"))){
	conf_path = malloc(strlen(cp) + strlen(SYSCONFDIR) + 2);
	sprintf(conf_path, "%s:" SYSCONFDIR, cp);
    } else {
	conf_path = strdup(SYSCONFDIR);
    }
    files = tchash_new(8, TC_LOCK_SLOPPY, 0);
    conf_init = 1;

    strcpy(cfg_file, getenv("HOME"));
    strcat(cfg_file, "/.tc2rc");
    tc2_add_config(cfg_file);

    return 0;
}

static int
do_include(void)
{
    void *s = NULL;
    char *inc;

    while(tcconf_nextvalue(top_conf, "TC2/include", &s, "%s", &inc) > 0){
	if(tchash_search(files, inc, -1, NULL, NULL)){
	    char *file = tcpath_find(inc, conf_path, R_OK);
	    if(file){
		tc2_add_config(file);
		free(file);
	    }
	    free(inc);
	}
    }

    return 0;
}

extern int
tc2_add_config(char *filename)
{
    if(!conf_init)
	init_config();

    top_conf = tcconf_load_file(top_conf, filename);
    do_include();

    return 0;
}

extern tcconf_section_t *
tc2_get_conf(char *sec)
{
    if(!conf_init)
	init_config();

    if(!top_conf)
	return NULL;

    return tcconf_getsection(top_conf, sec);
}

extern void
tc2_free_conf(void)
{
    if(top_conf)
	tcfree(top_conf);
    tchash_destroy(files, NULL);
    free(conf_path);
}
