/**
    Copyright (C) 2001, 2002, 2003  Michael Ahlberg, Måns Rullgård

    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and associated documentation
    files (the "Software"), to deal in the Software without
    restriction, including without limitation the rights to use, copy,
    modify, merge, publish, distribute, sublicense, and/or sell copies
    of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
    HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
    WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.
**/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "tc2.h"
#include "kernel.h"

static tchash_table_t *hooks;

extern int
tc2_init_hook(void)
{
    kprint("init", TC2_PRINT_DEBUG, "Hooks initialized\n");

    /* FIXME: initial size */
    hooks = tchash_new(100, TC_LOCK_STRICT, 0);

    return 0;
}

static void
free_hook(void *p)
{
    tclist_destroy(p, NULL);
}

extern void
tc2_free_hooks(void)
{
    tchash_destroy(hooks, free_hook);
}

extern int
tc2_call_hook(char *hook, void *opt)
{
    tclist_t *hl = NULL;
    tclist_item_t *li = NULL;
    int (*s)(void *);

    kprint("hook", TC2_PRINT_DEBUG, "Hook \"%s\" called\n", hook);

    tchash_find(hooks, hook, -1, &hl);

    if(hl){
	while((s = tclist_next(hl, &li)) != NULL) {
	    kprint("hook", TC2_PRINT_DEBUG + 1, "calling %p\n", s);
	    s(opt);
	}
    }

    return 0;
}

extern int
tc2_add_hook(char *hook, int (*fn)(void *))
{
    tclist_t *hl=NULL, *ret;
    int f;

    kprint("hook", TC2_PRINT_DEBUG, "add \"%s\"\n", hook);

    f=tchash_find(hooks, hook, -1, &hl);

    if(f!=0){
	hl = tclist_new(TC_LOCK_SLOPPY);
    }

    tclist_push(hl, fn);

    if(f!=0){
	tchash_search(hooks, hook, -1, hl, &ret);
    }
    
    return 0;
}

extern int
tc2_rm_hook(char *hook, int (*fn)(void *))
{
    tclist_t *hl = NULL;
    tclist_item_t *li = NULL;
    void *s;

    kprint("hook", TC2_PRINT_DEBUG, "remove \"%s\"\n", hook);

    tchash_find(hooks, hook, -1, &hl);

    if(hl){
	while((s = tclist_next(hl, &li)) != NULL) {
	    if(s==fn){
		kprint("hook", TC2_PRINT_DEBUG + 1, "removing %p\n", s);
		tclist_remove(hl, li, NULL);
	    }
	}
    }

    return 0;
}
