# Copyright (C) 2001-2004  Michael Ahlberg, Måns Rullgård

# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:

# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


# TC2_INIT
# Set up some things required by tc2.
# -----------------------------------
AC_DEFUN([TC2_INIT],
[AC_REQUIRE([AC_PROG_CC])
AC_REQUIRE([AC_PROG_INSTALL])
AC_REQUIRE([AC_PROG_MAKE_SET])
AC_REQUIRE([AC_ENABLE_SHARED])
AC_REQUIRE([AC_DISABLE_STATIC])
AC_REQUIRE([AC_LIBTOOL_DLOPEN])
AC_REQUIRE([AC_PROG_LIBTOOL])
AC_SUBST([moduledir], ["\$(pkglibdir)/modules"])
AC_SUBST([interfacedir], ["\$(pkgdatadir)/interfaces"])
AC_SUBST([PICFLAGS], [`echo $lt_lt_prog_compiler_pic | tr -d \"`])
AC_ARG_WITH(libtc, AC_HELP_STRING([--with-libtc=DIR],
				  [Prefix where libtc is installed.]),
	    [CFLAGS="$CFLAGS -I$withval/include"
	     LDFLAGS="$LDFLAGS -L$withval/lib"])
AC_ARG_WITH(tc2,
	    AC_HELP_STRING([--with-tc2=DIR], [Prefix where TC2 is installed.]),
            [AC_SUBST(TC2MAKE, $withval/bin/tc2make)
		CFLAGS="$CFLAGS -I$withval/include"
		LDFLAGS="$LDFLAGS -L$withval/lib"],
	    [AC_PATH_PROG(TC2MAKE, tc2make, no)])
if test "$TC2MAKE" = "no"; then
    AC_ERROR(TC2 not found.)
fi
AC_SUBST([TC2MAKE_FLAGS],["-I\$(top_srcdir)/interfaces -I$ac_aux_dir/interfaces"])
AC_ARG_ENABLE([verbose], [AC_HELP_STRING([--enable-verbose],[verbose build])],
	[silent=no],
	[silent=yes])
AM_CONDITIONAL([silent], [test $silent = yes])
patch -N <<\END
--- libtool~    Sat Jan 17 04:16:27 2004
+++ libtool     Sat Jan 17 04:23:37 2004
@@ -5993,6 +5993,8 @@
     if test -n "$current_libdirs"; then
       # Maybe just do a dry run.
       test -n "$run" && current_libdirs=" -n$current_libdirs"
+      # pass --quiet recursively if set in this libtool invocation
+      test "$show" = ":" && current_libdirs=" --quiet$current_libdirs"
       exec_cmd='$SHELL $[]0 --finish$current_libdirs'
     else
       exit 0
END
]) # TC2_INIT

# TC2_ENABLE_MODULE
# Enable building of the module currently being tested.
# -----------------------------------------------------
AC_DEFUN([TC2_ENABLE_MODULE],
[eval $_tc2_modlist="\"\$$_tc2_modlist $_tc2_moddir\""
eval _tc2_enable_${_tc2_module}=yes
])

# TC2_ADD_LDFLAGS(FLAGS)
# Add FLAGS to the linker flags of the current module.
# ----------------------------------------------------
AC_DEFUN([TC2_ADD_LDFLAGS],
[eval ${_tc2_module}_MODLIBS="\"\$${_tc2_module}_MODLIBS $1\""
])

# TC2_ADD_INC(FLAGS)
# Add FLAGS to the preprocessor flags of the current module.
# ----------------------------------------------------
AC_DEFUN([TC2_ADD_INC],
[eval ${_tc2_module}_MODINC="\"\$${_tc2_module}_MODINC $1\""
])

# TC2_CHECK_LIB(NAME, LIB, FUNC, [OTHER-LIBRARIES], [HEADERS], [INCLUDES],
#		[ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
# Check if FUNC is present in LIB.  If found, add -llib to the linker
# flags of the current module.  If not empty, check HEADERS.
# -------------------------------------------------------------------
AC_DEFUN([TC2_CHECK_LIB],
[tc2_check_lib=yes
AC_ARG_WITH([$1], [AC_HELP_STRING([--with-$1=DIR], [$1 installed in DIR])],
	    [TC2_ADD_LDFLAGS([-L$withval/lib])
	     TC2_ADD_INC([-I$withval/include])])
AC_CHECK_LIB([$2], [$3],
	     [TC2_ADD_LDFLAGS([-l$2 $4])],
	     [AC_MSG_WARN([$1 not found.])
	      tc2_check_lib=no],
	     [$4 `eval "echo \\\$${_tc2_module}_MODLIBS"`])
ifelse($5, [], [], [AC_CHECK_HEADERS([$5], [], [tc2_check_lib=no], [$6])])
if test x$tc2_check_lib = xyes; then
	TC2_ADD_INC([-D[]AS_TR_CPP(HAVE_LIB$2)])
	$7
else
	true
	$8
fi
])

define([TC2_PAD6], [[$1], [$2], [$3], [$4], [$5], [$6]])
define([TC2_ARG1], [$1])

# TC2_CHECK_LIBS([NAME, LIB, FUNC, OTHER-LIBRARIES, HEADERS, INCLUDES], ...)
# Check for libraries and enable a module if all are present.
# Can only be used in configure.ac.in fragments
# ---------------------------------------------------------------------
AC_DEFUN([TC2_CHECK_LIBS],
[ifelse([$1], [], [TC2_ENABLE_MODULE],
[TC2_CHECK_LIB(TC2_PAD6($1), [TC2_CHECK_LIBS(m4_shift($@))],
	       [AC_MSG_WARN([ifelse(,,[TC2_ARG1($1)]) not found.])])])
]) # TC2_CHECK_LIBS

# TC2_CHECK_INTERFACE(NAME, [ACTION-IF-FOUND], [ACTION-IF-NOT-FOUND])
# Check for the presence of TC2 interface NAME.
# -------------------------------------------------------------------
AC_DEFUN([TC2_CHECK_INTERFACE],
[AC_CACHE_CHECK([for interface $1], tc2_cv_interface_$1,
    [tc2_cv_interface_$1=no
     $TC2MAKE --query=$1 >/dev/null 2>/dev/null && tc2_cv_interface_$1=yes])
if test $tc2_cv_interface_$1 = yes; then
    :
    $2
else
    :
    $3
fi
])
